/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.shopping.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.common.constant.CacheBizConstant;
import org.springblade.common.utils.CommonUtil;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.shopping.entity.Consign;
import org.springblade.modules.shopping.service.IConsignService;
import org.springblade.modules.shopping.wrapper.ConsignConfigWrapper;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.shopping.entity.ConsignConfig;
import org.springblade.modules.shopping.vo.ConsignConfigVO;
import org.springblade.modules.shopping.service.IConsignConfigService;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.List;

/**
 * 寄售配置表 控制器
 *
 * @author BladeX
 * @since 2022-02-13
 */
@RestController
@AllArgsConstructor
@RequestMapping("shopping/consignconfig")
@Api(value = "寄售配置表", tags = "寄售配置表接口")
public class ConsignConfigController extends BladeController {

	private final IConsignConfigService consignConfigService;
	private final BladeRedis bladeRedis;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入consignConfig")
	public R<ConsignConfigVO> detail(ConsignConfig consignConfig) {
		ConsignConfig detail = consignConfigService.getOne(Condition.getQueryWrapper(consignConfig));
		return R.data(ConsignConfigWrapper.build().entityVO(detail));
	}

	/**
	 * 分页 寄售配置表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入consignConfig")
	public R<IPage<ConsignConfigVO>> list(ConsignConfig consignConfig, Query query) {
		IPage<ConsignConfig> pages = consignConfigService.page(Condition.getPage(query),
			Condition.getQueryWrapper(consignConfig)
				.orderByDesc("is_enabled","is_now", "start_time"));
		return R.data(ConsignConfigWrapper.build().pageVO(pages));
	}

	@GetMapping("/selectList")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入consignConfig")
	public R<List<ConsignConfig>> selectList(ConsignConfig consignConfig) {
		consignConfig.setIsNow(2);
		return R.data(consignConfigService.list(Condition.getQueryWrapper(consignConfig)));
	}

	@GetMapping("/getList")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入consignConfig")
	public R<IPage<ConsignConfigVO>> getList(ConsignConfig consignConfig, Query query) {
		consignConfig.setIsEnabled(1);
		IPage<ConsignConfig> pages = consignConfigService.page(Condition.getPage(query),
			Condition.getQueryWrapper(consignConfig)
				.orderByDesc("start_time"));
		return R.data(ConsignConfigWrapper.build().pageVO(pages));
	}

	@GetMapping("/payList")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "支付批次", notes = "传入consignConfig")
	public R<IPage<ConsignConfigVO>> payList(ConsignConfig consignConfig, Query query) {
		IPage<ConsignConfig> pages = consignConfigService.page(Condition.getPage(query),
			Condition.getQueryWrapper(consignConfig)
				.eq("is_enabled","1")
				.last("start_time"));
		return R.data(ConsignConfigWrapper.build().pageVO(pages));
	}

	/**
	 * 自定义分页 寄售配置表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入consignConfig")
	public R<IPage<ConsignConfigVO>> page(ConsignConfigVO consignConfig, Query query) {
		IPage<ConsignConfigVO> pages = consignConfigService.selectConsignConfigPage(Condition.getPage(query), consignConfig);
		return R.data(pages);
	}

	/**
	 * 新增 寄售配置表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入consignConfig")
	public R save(@Valid @RequestBody ConsignConfig consignConfig) {
		//清空缓存
		bladeRedis.del(CacheBizConstant.CACHE_CONSIGN_CONFIG);
		consignConfig.setConfigNo(CommonUtil.genTimeID());
		return R.status(consignConfigService.save(consignConfig));
	}

	/**
	 * 修改 寄售配置表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入consignConfig")
	public R update(@Valid @RequestBody ConsignConfig consignConfig) {
		//清空缓存
		bladeRedis.del(CacheBizConstant.CACHE_CONSIGN_CONFIG);
		return R.status(consignConfigService.updateById(consignConfig));
	}

	/**
	 * 新增或修改 寄售配置表
	 */
	@ApiLog("新增或修改场次配置")
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入consignConfig")
	public R submit(@Valid @RequestBody ConsignConfig consignConfig) {
		//清空缓存
		bladeRedis.del(CacheBizConstant.CACHE_CONSIGN_CONFIG);
		boolean flg = consignConfigService.saveOrUpdate(consignConfig);
		if (consignConfig.getId() != null){
			//修改寄售表开始和结束时间
			ConsignConfig query = consignConfigService.getById(consignConfig.getId());
			consignConfigService.updateConsignTime(query);
		}
		return R.status(flg);
	}


	/**
	 * 删除 寄售配置表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(consignConfigService.removeByIds(Func.toLongList(ids)));
	}


}
